#!/usr/bin/python

"""
radio.py 0.3

A script that makes it easy to listen to online radio via mplayer

"""


###########################################################################
 #   Copyright (C) 2007 by Guy Rutenberg                                   #
 #   guyrutenberg@gmail.com                                                #
 #                                                                         #
 #   This program is free software; you can redistribute it and/or modify  #
 #   it under the terms of the GNU General Public License as published by  #
 #   the Free Software Foundation; either version 2 of the License, or     #
 #   (at your option) any later version.                                   #
 #                                                                         #
 #   This program is distributed in the hope that it will be useful,       #
 #   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
 #   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
 #   GNU General Public License for more details.                          #
 #                                                                         #
 #   You should have received a copy of the GNU General Public License     #
 #   along with this program; if not, write to the                         #
 #   Free Software Foundation, Inc.,                                       #
 #   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

import sys
import os
import subprocess
import time
from optparse import OptionParser

radiostations = {#name:(address,homepage),
    'Galgalatz':('http://gifs.msn.co.il/media/gglz.asx','http://glz.msn.co.il/'),
    'Radio Tel Aviv':('mms://s35wm.castup.net/992120001-52.wmv','http://www.102fm.co.il/'),
    'Galatz':('http://gifs.msn.co.il/media/glz.asx','http://glz.msn.co.il/'),
    'Reshet Bet':('mms://s35wm.castup.net/990310001-52.wmv','http://bet.iba.org.il/'),
    'Reshet Gimel':('mms://s36wm.castup.net/990310004-52.wmv','http://bet.iba.org.il/'),
    '88fm':('mms://s35wm.castup.net/990310006-52.wmv','http://88fm.iba.org.il/'),
    'BBC World Service':('http://www.bbc.co.uk/worldservice/meta/tx/nb/live_news_au_nb.ram','http://www.bbc.co.uk/worldservice/'),
    'BBC1':('http://www.bbc.co.uk/radio1/realaudio/media/r1live.ram','http://www.bbc.co.uk/radio1/'),
    'BBC2':('http://www.bbc.co.uk/radio2/realmedia/fmg2.ram','http://www.bbc.co.uk/radio2/'),
    'BBC3':('http://www.bbc.co.uk/radio3/ram/r3g2.ram','http://www.bbc.co.uk/radio3/'),
    'BBC4':('http://www.bbc.co.uk/radio4/realplayer/media/fmg2.ram','http://www.bbc.co.uk/radio4/'),
    'BBC5':('http://www.bbc.co.uk/fivelive/live/surestream_int.ram','http://www.bbc.co.uk/fivelive/'),
    'KING-FM':('http://www.king.org/kingfm.asx','http://www.king.org/'),
    'CNN':('http://www.cnn.com/audio/radio/liveaudio.asx','http://www.cnn.com/'),
    '103fm':('http://live.103.fm/103fm-high/','http://www.103.fm/'),
    'Radius':('http://s34wm.castup.net/993820002-52.wmv','http://www.100fm.co.il/'),
    }

def parseArguments():
    parser = OptionParser(usage="%prog [options] station", 
                        version="%prog 0.3")
    parser.add_option("-s", "--sleep", dest="sleep", type="int", default=0,
                    help="go to sleep after MIN minutes", metavar="MIN")
    parser.add_option("-w", "--wake-up", dest="wake", type="int", default=0,
                    help="wake up and start playing after MIN minutes", metavar="MIN")
    parser.add_option("-l", "--list", dest="list", action="store_true", default=False,
                    help="show a list of known radio stations and their homepage")
    parser.add_option("-c", "--cache", dest="cache", type="int", default=128,
                    help="set the size of the cache in KBytes [default: %default]", metavar="SIZE")
    
    return parser.parse_args()

def main(options, args):
    if options.list:
        showStations()
        return 0
    
    if len(args)<1:
        print "you need to supply a station name"
        showStations()
        return 1
    
    try:
        radiostations[args[0]]
    except KeyError:
        print "Didn't recognized this station name:",args[0]
        showStations()
        return 1
    
    if options.wake > 0:
        print "Radio will wake up in", options.wake, "minutes"
        time.sleep(60*options.wake)
    
    if options.cache > 32: #mplayer requires cache>=32
        cache = str(options.cache)
    else: cache = str(32)
    
    #check if we need the '-playlist' argument for mplayer
    playlist=''
    if radiostations[args[0]][0][-4:] in ('.asx','.ram'):
        playlist='-playlist'
    
    execargs=['mplayer', '-softvol', '-cache', cache, playlist, radiostations[args[0]][0] ]
    if options.sleep > 0:
        print "Radio will go to sleep in", options.sleep, "minutes"
        pid = subprocess.Popen(execargs).pid
        time.sleep(60*options.sleep)
        os.kill(pid, 15) # 15 = SIGTERM
    else: #in case we don't sleep then start the player without forking
        os.execvp(execargs[0],execargs)

def showStations():
    keys = radiostations.keys()
    keys.sort()
    
    maxlen = 0 # find the longest station name
    for key in keys:
        if len(key) > maxlen: maxlen = len(key) 
    for key in keys:
        print key.ljust(maxlen+1), radiostations[key][1]

if __name__== '__main__':
    (options, args) = parseArguments()
    sys.exit(main(options, args))
